@@ -1,5 +1,5 @@
 #!/usr/bin/perl
-# $Id: bench.pl,v 1.10 2006/04/29 00:35:08 tinita Exp $
+# $Id: bench.pl,v 1.12 2006/07/13 18:05:27 tinita Exp $
 use strict;
 use warnings;
 use lib qw(blib/lib ../blib/lib);
@@ -8,6 +8,7 @@ my $count = 0;
 my $ht_file = 'test.htc';
 #$ht_file = 'test.htc.10';
 #$ht_file = 'test.htc.20';
+my $htcc_file = $ht_file . 'c';
 my $tt_file = "test.tt";
 #$tt_file = "examples/test.tt.10";
 #$tt_file = "examples/test.tt.20";
@@ -18,18 +19,19 @@ mkdir "cache/jit";
 my %use = (
 	'HTML::Template'           => 0,
 	'HTML::Template::Compiled' => 0,
+	'HTML::Template::Compiled::Classic' => 0,
 	'HTML::Template::JIT'      => 0,
 	'Template'                 => 0,
 	# not yet
 	'Text::ScriptTemplate'     => 0,
 );
-for my $key (keys %use) {
+for my $key (sort keys %use) {
 	eval "require $key";
 	$use{$key} = 1 unless $@;
 	my $version = $use{$key} ? $key->VERSION : "-";
-    printf "using %25s %s\n", $key, $version;
+    printf "using %35s %s\n", $key, $version;
 }
-$HTML::Template::Compiled::NEW_CHECK = 10;
+HTML::Template::Compiled->ExpireTime(10);
 use Benchmark;
 my $debug = 0;
 $ENV{'HTML_TEMPLATE_ROOT'} = "examples";
@@ -39,11 +41,10 @@ my $LOOP_CONTEXT   = 1;
 my $GLOBAL_VARS    = 0;
 my $CASE_SENSITIVE = 1;
 sub new_htc {
-	my $t1 = HTML::Template::Compiled->new(
+	my $t1 = HTML::Template::Compiled->new_file( $ht_file,
 		#path => 'examples',
 		case_sensitive => $CASE_SENSITIVE, # slow down
 		loop_context_vars => $LOOP_CONTEXT,
-		filename => $ht_file,
 		debug => $debug,
 		# note that you have to create the cachedir
 		# first, otherwise it will run without cache
@@ -52,10 +53,26 @@ sub new_htc {
 		out_fh => 1,
         global_vars => $GLOBAL_VARS,
 	);
-	#my $size = total_size($t1);
-	#print "size htc = $size\n";
 	return $t1;
 }
+
+sub new_htcc {
+	my $t1 = HTML::Template::Compiled::Classic->new_file( $htcc_file,
+		#path => 'examples',
+		case_sensitive => $CASE_SENSITIVE, # slow down
+		loop_context_vars => $LOOP_CONTEXT,
+		debug => $debug,
+		# note that you have to create the cachedir
+		# first, otherwise it will run without cache
+        cache_dir => ($FILE_CACHE ? "cache/htc" : undef),
+        cache => $MEM_CACHE,
+		out_fh => 1,
+        global_vars => $GLOBAL_VARS,
+        debug => 0,
+	);
+	return $t1;
+}
+
 sub new_tst {
 	my $t = Text::ScriptTemplate->new();
 	$t->load($tst_file);
@@ -63,6 +80,7 @@ sub new_tst {
 	#print "size htc = $size\n";
 	return $t;
 }
+
 sub new_ht {
 	my $t2 = HTML::Template->new(
 		case_sensitive => $CASE_SENSITIVE, # slow down
@@ -75,10 +93,9 @@ sub new_ht {
         file_cache_dir => 'cache/ht') : (),
         global_vars => $GLOBAL_VARS,
 	);
-	#my $size = total_size($t2);
-	#print "size ht  = $size\n";
 	return $t2;
 }
+
 sub new_htj {
 	my $t2 = HTML::Template::JIT->new(
 		loop_context_vars => 1,
@@ -90,6 +107,7 @@ sub new_htj {
 	);
 	return $t2;
 }
+
 sub new_tt {
 	my $tt= Template->new(
 		COMPILE_EXT => '.ttc',
@@ -106,6 +124,7 @@ sub new_st {
 	my $st = Text::ScriptTemplate->new;
 	$st->load("examples/template.st");
 }
+
 my %params = (
 	name => '',
 	loopa => [{a=>3},{a=>4},{a=>5}],
@@ -125,6 +144,7 @@ my %params = (
 );
 open OUT, ">>/dev/null";
 #open OUT, ">&STDOUT";
+
 sub output {
 	my $t = shift;
 	return unless defined $t;
@@ -138,6 +158,7 @@ sub output {
 	#print "size $t = $size\n";
 	#print "\nOUT: $out";
 }
+
 #open TT_OUT, ">&STDOUT";
 sub output_tst {
 	my $t = shift;
@@ -162,19 +183,28 @@ sub output_tt {
 }
 
 my $global_htc = $use{'HTML::Template::Compiled'} ? new_htc : undef;
+my $global_htcc = $use{'HTML::Template::Compiled::Classic'} ? new_htcc : undef;
 my $global_ht = $use{'HTML::Template'} ? new_ht : undef;
 my $global_htj = $use{'HTML::Template::JIT'} ? new_htj : undef;
 my $global_tt = $use{'Template'} ? new_tt : undef;
 my $global_tst = $use{'Text::ScriptTemplate'} ? new_tst : undef;
 if(1) {
 timethese ($ARGV[0]||-1, {
-		$use{'HTML::Template::Compiled'} ? (
+        $use{'HTML::Template::Compiled'} ? (
             # deactivate memory cache
             #new_htc_w_clear_cache => sub {my $t = new_htc();$t->clear_cache},
             # normal, with memory cache
-						#new_htc => sub {my $t = new_htc()},
-						#output_htc => sub {output($global_htc)},
-						all_htc => sub {my $t = new_htc();output($t)},
+            #new_htc => sub {my $t = new_htc()},
+            #output_htc => sub {output($global_htc)},
+            all_htc => sub {my $t = new_htc();output($t)},
+        ) : (),
+        $use{'HTML::Template::Compiled::Classic'} ? (
+            # deactivate memory cache
+            #new_htc_w_clear_cache => sub {my $t = new_htc();$t->clear_cache},
+            # normal, with memory cache
+            #new_htc => sub {my $t = new_htc()},
+            #output_htc => sub {output($global_htc)},
+            all_htcc => sub {my $t = new_htcc();output($t)},
         ) : (),
 		$use{'HTML::Template'} ? (
 			#new_ht => sub {my $t = new_ht()},
@@ -190,12 +220,8 @@ timethese ($ARGV[0]||-1, {
 					#new_tt => sub {my $t = new_tt();},
 					#output_tt => sub {output_tt($global_tt)},
 						process_tt => sub {output_tt($global_tt)},
-        ): (),
-#        $use{'Template'} ? (
-#					#new_tt => sub {my $t = new_tt();},
-#					#output_tt => sub {output_tt($global_tt)},
 #						all_tt_new_object => sub {my $t = new_tt();output_tt($t)},
-#        ): (),
+        ): (),
         $use{'Text::ScriptTemplate'} ? (
 					#new_tst => sub {my $t = new_tst();},
                     #output_tst => sub {output_tst($global_tst)},
